# flake8: noqa
"""scripts generates large amount of timers for load testing
"""

import os
import sys
from pathlib import Path

myauth_dir = Path(__file__).parent.parent.parent.parent.parent / "myauth"
sys.path.insert(0, str(myauth_dir))

import django
from django.apps import apps

# init and setup django project
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "myauth.settings.local")
django.setup()

"""SCRIPT"""
import datetime as dt
import random
from pathlib import Path

# from django.contrib.auth.models import User
from django.utils.timezone import now
from eveuniverse.models import EveSolarSystem, EveType

from allianceauth.eveonline.models import EveCorporationInfo

from structuretimers import constants
from structuretimers.models import Timer

MAX_TIMERS = 20

structure_type_ids = EveType.objects.filter(
    eve_group__eve_category_id=constants.EVE_CATEGORY_ID_STRUCTURE,
    published=True,
).values_list("id", flat=True)
eve_solar_system_ids = EveSolarSystem.objects.values_list("id", flat=True)
owner_names = EveCorporationInfo.objects.values_list("corporation_name", flat=True)
for _ in range(MAX_TIMERS):
    Timer.objects.create(
        timer_type=random.choice([elem[0] for elem in Timer.Type.choices]),
        eve_solar_system_id=random.choice(eve_solar_system_ids),
        structure_type_id=random.choice(structure_type_ids),
        date=now()
        + dt.timedelta(days=random.randint(1, 100), hours=random.randint(0, 59)),
        structure_name=f"Generated Timer",
        objective=random.choice([elem[0] for elem in Timer.Objective.choices]),
        owner_name=random.choice(owner_names),
    )

print("DONE")



import math
from copy import deepcopy
from datetime import timedelta

from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.core.exceptions import PermissionDenied
from django.shortcuts import get_object_or_404
from django.urls import reverse, reverse_lazy
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.timezone import now
from django.utils.translation import gettext as _
from django.views import View
from django.views.generic import (
    CreateView,
    DeleteView,
    ListView,
    TemplateView,
    UpdateView,
)
from django.views.generic.detail import BaseDetailView
from eveuniverse.models import EveSolarSystem, EveType

from allianceauth.eveonline.evelinks import dotlan
from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag
from app_utils.messages import messages_plus
from app_utils.views import (
    JSONResponseMixin,
    bootstrap_label_html,
    fontawesome_link_button_html,
    link_html,
    no_wrap_html,
    yesno_str,
)

from . import __title__
from .app_settings import (
    STRUCTURETIMERS_DEFAULT_PAGE_LENGTH,
    STRUCTURETIMERS_PAGING_ENABLED,
)
from .constants import (
    EVE_CATEGORY_ID_STRUCTURE,
    EVE_GROUP_ID_CONTROL_TOWER,
    EVE_GROUP_ID_MOBILE_DEPOT,
    EVE_TYPE_ID_CUSTOMS_OFFICE,
    EVE_TYPE_ID_IHUB,
    EVE_TYPE_ID_TCU,
)
from .forms import TimerForm
from .models import DistancesFromStaging, StagingSystem, Timer

logger = LoggerAddTag(get_extension_logger(__name__), __title__)
DATETIME_FORMAT = "%Y-%m-%d %H:%M"
MAX_HOURS_PASSED = 2


class TimerListView(LoginRequiredMixin, PermissionRequiredMixin, TemplateView):
    template_name = "structuretimers/timer_list.html"
    permission_required = "structuretimers.basic_access"

    def get_context_data(self, **kwargs):
        staging_systems_qs = StagingSystem.objects.select_related(
            "eve_solar_system", "eve_solar_system__eve_constellation__eve_region"
        )
        try:
            selected_staging_system = staging_systems_qs.get(
                eve_solar_system__name=self.request.GET.get("staging")
            )
        except (StagingSystem.DoesNotExist, ValueError):
            selected_staging_system = staging_systems_qs.filter(is_main=True).first()
            if not selected_staging_system:
                selected_staging_system = staging_systems_qs.first()
        stageing_systems = staging_systems_qs.order_by("eve_solar_system__name")
        context = super().get_context_data(**kwargs)
        context.update(
            {
                "current_time": now().strftime("%Y-%m-%d %H:%M:%S"),
                "max_hours_expired": MAX_HOURS_PASSED,
                "title": __title__,
                "data_tables_page_length": STRUCTURETIMERS_DEFAULT_PAGE_LENGTH,
                "data_tables_paging": STRUCTURETIMERS_PAGING_ENABLED,
                "selected_staging_system": selected_staging_system,
                "stageing_systems": stageing_systems,
            }
        )
        return context


class TimerListDataView(
    LoginRequiredMixin, PermissionRequiredMixin, JSONResponseMixin, ListView
):
    """Produce timer list in JSON for AJAX call."""

    model = Timer
    permission_required = "structuretimers.basic_access"

    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)

    def get_queryset(self):
        qs = super().get_queryset()
        timers_qs = qs.visible_to_user(self.request.user)
        tab_name = self.kwargs.get("tab_name")
        if tab_name == "current":
            timers_qs = timers_qs.filter(
                date__gte=now() - timedelta(hours=MAX_HOURS_PASSED)
            )
        else:
            timers_qs = timers_qs.filter(date__lt=now())

        timers_qs = timers_qs.select_related(
            "eve_solar_system__eve_constellation__eve_region",
            "structure_type",
            "structure_type__eve_group",
            "eve_character",
            "eve_corporation",
            "eve_alliance",
        )
        return timers_qs

    def get_data(self, context):
        staging_system_pk = self.request.GET.get("staging")
        if staging_system_pk:
            distances_map = {
                obj.timer_id: obj
                for obj in DistancesFromStaging.objects.filter(
                    staging_system__pk=staging_system_pk
                ).all()
            }
        else:
            distances_map = dict()
        data = list()
        for timer in self.object_list:
            # location
            location = link_html(
                dotlan.solar_system_url(timer.eve_solar_system.name),
                timer.eve_solar_system.name,
            )
            if timer.location_details:
                location += format_html("<br><em>{}</em>", timer.location_details)

            location += format_html(
                "<br>{}", timer.eve_solar_system.eve_constellation.eve_region.name
            )
            # distance
            try:
                distances = distances_map[timer.id]
            except KeyError:
                distance_text = "?"
                distances = None
            else:
                light_years_text = (
                    f"{math.ceil(distances.light_years * 10) / 10} ly"
                    if distances.light_years is not None
                    else "N/A"
                )
                jumps_text = (
                    f"{distances.jumps} jumps" if distances.jumps is not None else "N/A"
                )
                distance_text = format_html("{}<br>{}", light_years_text, jumps_text)

            # structure & timer type & fitting image
            timer_type = bootstrap_label_html(
                timer.get_timer_type_display(), timer.label_type_for_timer_type()
            )
            if timer.structure_type:
                structure_type_icon_url = timer.structure_type.icon_url(size=64)
                structure_type_name = timer.structure_type.name
            else:
                structure_type_icon_url = ""
                structure_type_name = "(unknown)"

            structure = format_html(
                '<div class="flex-container">'
                '  <div style="padding-top: 4px;"><img src="{}" width="40"></div>'
                '  <div style="text-align: left;">'
                "    {}&nbsp;<br>"
                "    {}"
                "  </div>"
                "</div>",
                structure_type_icon_url,
                mark_safe(bootstrap_label_html(structure_type_name, "info")),
                timer_type,
            )

            # objective & tags
            tags = list()
            is_restricted = False
            if timer.is_opsec:
                tags.append(bootstrap_label_html("OPSEC", "danger"))
                is_restricted = True

            if timer.visibility != Timer.Visibility.UNRESTRICTED:
                tags.append(
                    bootstrap_label_html(timer.get_visibility_display(), "info")
                )
                is_restricted = True

            if timer.is_important:
                tags.append(bootstrap_label_html("Important", "warning"))

            objective = format_html(
                "{}<br>{}",
                mark_safe(
                    bootstrap_label_html(
                        timer.get_objective_display(), timer.label_type_for_objective()
                    )
                ),
                mark_safe(" ".join(tags)),
            )

            # name & owner
            if timer.owner_name:
                owner_name = timer.owner_name
                owner = owner_name
            else:
                owner = "-"
                owner_name = ""

            structure_name = timer.structure_name if timer.structure_name else "-"
            name = format_html("{}<br>{}", structure_name, owner)

            if timer.eve_corporation:
                corporation_name = timer.eve_corporation.corporation_name
            else:
                corporation_name = "-"

            # creator
            # if timer.eve_character:
            #     creator = format_html(
            #         "{}<br>{}",
            #         link_html(
            #             evewho.character_url(timer.eve_character.character_id),
            #             timer.eve_character.character_name,
            #         ),
            #         corporation_name,
            #     )
            # elif corporation_name:
            #     creator = corporation_name
            # else:
            #     creator = "-"

            # visibility
            visibility = ""
            if timer.visibility == Timer.Visibility.ALLIANCE and timer.eve_alliance:
                visibility = timer.eve_alliance.alliance_name
            elif timer.visibility == Timer.Visibility.CORPORATION:
                visibility = corporation_name

            data.append(
                {
                    "id": timer.id,
                    "local_time": timer.date.isoformat(),
                    "date": timer.date.isoformat(),
                    "location": location,
                    "structure_details": structure,
                    "name_objective": name,
                    "owner": objective,
                    # "creator": creator,
                    "distance": distance_text,
                    "distance_light_years": distances.light_years
                    if distances
                    else None,
                    "distance_jumps": distances.jumps if distances else None,
                    "actions": self._get_data_actions(timer),
                    "timer_type_name": timer.get_timer_type_display(),
                    "objective_name": timer.get_objective_display(),
                    "system_name": timer.eve_solar_system.name,
                    "region_name": timer.eve_solar_system.eve_constellation.eve_region.name,
                    "structure_type_name": timer.structure_type.name,
                    "owner_name": owner_name,
                    "visibility": visibility,
                    "opsec_str": yesno_str(timer.is_opsec),
                    "is_opsec": timer.is_opsec,
                    "is_passed": timer.date < now(),
                    "is_important": timer.is_important,
                    "is_restricted": is_restricted,
                }
            )

        return data

    def _get_data_actions(self, timer):
        actions = ""
        if timer.details_image_url or timer.details_notes:
            disabled_html = ""
            button_type = "primary"
            data_toggle = 'data-toggle="modal" data-target="#modalTimerDetails" '
            title = "Show details of this timer"
        else:
            button_type = "default"
            disabled_html = ' disabled="disabled"'
            data_toggle = ""
            title = "No details available"
        actions += (
            format_html(
                '<a type="button" id="timerboardBtnDetails" '
                f'class="btn btn-{button_type}" title="{title}"'
                f"{data_toggle}"
                f'data-timerpk="{timer.pk}"{disabled_html}>'
                '<i class="fas fa-search-plus"></i></a>'
            )
            + "&nbsp;"
        )
        if timer.user_can_edit(self.request.user):
            actions += (
                fontawesome_link_button_html(
                    reverse("structuretimers:delete", args=(timer.pk,)),
                    "far fa-trash-alt",
                    "danger",
                    "Delete this timer",
                )
                + "&nbsp;"
                + fontawesome_link_button_html(
                    reverse("structuretimers:edit", args=(timer.pk,)),
                    "far fa-edit",
                    "warning",
                    "Edit this timer",
                )
            )
        if self.request.user.has_perm("structuretimers.create_timer"):
            actions += "&nbsp;" + fontawesome_link_button_html(
                reverse("structuretimers:copy", args=(timer.pk,)),
                "far fa-copy",
                "success",
                "Copy this timer",
            )
        return no_wrap_html(actions)


class TimerDetailDataView(
    LoginRequiredMixin, PermissionRequiredMixin, JSONResponseMixin, BaseDetailView
):
    permission_required = "structuretimers.basic_access"
    model = Timer

    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.visible_to_user(self.request.user).select_related(
            "structure_type", "eve_solar_system"
        )

    def get_context_data(self, object):
        context = {
            "display_name": str(object),
            "structure_display_name": object.structure_display_name,
            "date": object.date.strftime(DATETIME_FORMAT),
            "details_image_url": object.details_image_url
            if object.details_image_url
            else "",
            "notes": object.details_notes if object.details_notes else "",
        }
        return context


class TimerManagementView(LoginRequiredMixin, PermissionRequiredMixin, View):
    index_redirect = "structuretimers:timer_list"
    success_url = reverse_lazy(index_redirect)
    model = Timer
    form_class = TimerForm
    title = _("Edit Structure Timer")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["title"] = self.title
        return context


class AddUpdateMixin:
    def get_form_kwargs(self):
        """
        Inject the request user into the kwargs passed to the form
        """
        kwargs = super().get_form_kwargs()
        kwargs.update({"user": self.request.user})
        return kwargs


class CreateTimerView(TimerManagementView, AddUpdateMixin, CreateView):
    template_name_suffix = "_create_form"
    permission_required = (
        "structuretimers.basic_access",
        "structuretimers.create_timer",
    )
    title = _("Create New Timer")

    def form_valid(self, form):
        result = super().form_valid(form)
        timer = self.object
        logger.info(
            "Created new timer in %s at %s by user %s",
            timer.eve_solar_system,
            timer.date,
            self.request.user,
        )
        messages_plus.success(
            self.request,
            _("Added new timer for %(type)s in %(system)s at %(time)s.")
            % {
                "type": timer.structure_type.name,
                "system": timer.eve_solar_system.name,
                "time": timer.date.strftime(DATETIME_FORMAT),
            },
        )
        return result


class EditTimerMixin:
    permission_required = "structuretimers.basic_access"

    def dispatch(self, request, *args, **kwargs):
        response = super().dispatch(request, *args, **kwargs)
        if response.status_code == 200:
            if (
                not self.object.user_can_edit(self.request.user)
                or not Timer.objects.filter(pk=self.object.pk)
                .visible_to_user(self.request.user)
                .exists()
            ):
                raise PermissionDenied()

        return response


class EditTimerView(EditTimerMixin, TimerManagementView, AddUpdateMixin, UpdateView):
    template_name_suffix = "_update_form"


class CopyTimerView(CreateTimerView):
    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        old_obj = get_object_or_404(Timer, pk=self.kwargs["pk"])
        new_obj = deepcopy(old_obj)
        new_obj.pk = None
        new_obj.date = None
        kwargs["instance"] = deepcopy(new_obj)
        return kwargs


class RemoveTimerView(EditTimerMixin, TimerManagementView, DeleteView):
    pass


class Select2SolarSystemsView(JSONResponseMixin, TemplateView):
    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)

    def get_context_data(self):
        term = self.request.GET.get("term")
        if term:
            results = [
                {"id": row["id"], "text": row["name"]}
                for row in EveSolarSystem.objects.filter(name__istartswith=term).values(
                    "id", "name"
                )
            ]
        else:
            results = None

        return {"results": results}


class Select2StructureTypesView(JSONResponseMixin, TemplateView):
    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)

    def get_context_data(self):
        term = self.request.GET.get("term")
        if term:
            types_qs = (
                EveType.objects.filter(
                    eve_group__eve_category_id=EVE_CATEGORY_ID_STRUCTURE, published=True
                )
                | EveType.objects.filter(
                    eve_group_id__in=[
                        EVE_GROUP_ID_CONTROL_TOWER,
                        EVE_GROUP_ID_MOBILE_DEPOT,
                    ],
                    published=True,
                )
                | EveType.objects.filter(
                    id__in=[
                        EVE_TYPE_ID_CUSTOMS_OFFICE,
                        EVE_TYPE_ID_IHUB,
                        EVE_TYPE_ID_TCU,
                    ]
                )
            )
            types_qs = (
                types_qs.select_related("eve_category", "eve_category__eve_group")
                .distinct()
                .filter(name__icontains=term)
            )
            results = [
                {"id": row["id"], "text": row["name"]}
                for row in types_qs.values("id", "name")
            ]
        else:
            results = None

        return {"results": results}



# Generated by Django 3.1.12 on 2021-07-13 19:24

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveonline", "0014_auto_20210105_1413"),
        ("eveuniverse", "0005_type_materials_and_sections"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("structuretimers", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="schedulednotification",
            name="notification_rule",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="scheduled_notifications",
                to="structuretimers.notificationrule",
            ),
        ),
        migrations.AlterField(
            model_name="schedulednotification",
            name="timer",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="scheduled_notifications",
                to="structuretimers.timer",
            ),
        ),
        migrations.AlterField(
            model_name="timer",
            name="eve_alliance",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Alliance of the user who created this timer",
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="eveonline.eveallianceinfo",
            ),
        ),
        migrations.AlterField(
            model_name="timer",
            name="eve_character",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Main character of the user who created this timer",
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="eveonline.evecharacter",
            ),
        ),
        migrations.AlterField(
            model_name="timer",
            name="eve_corporation",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Corporation of the user who created this timer",
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="eveonline.evecorporationinfo",
            ),
        ),
        migrations.AlterField(
            model_name="timer",
            name="eve_solar_system",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eveuniverse.evesolarsystem",
            ),
        ),
        migrations.AlterField(
            model_name="timer",
            name="structure_type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eveuniverse.evetype",
            ),
        ),
        migrations.AlterField(
            model_name="timer",
            name="user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.CreateModel(
            name="StagingSystem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("is_main", models.BooleanField(default=False)),
                (
                    "eve_solar_system",
                    models.OneToOneField(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        related_name="+",
                        to="eveuniverse.evesolarsystem",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DistancesFromStaging",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("light_years", models.FloatField(blank=True, default=None, null=True)),
                (
                    "jumps",
                    models.PositiveIntegerField(blank=True, default=None, null=True),
                ),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "staging_system",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="distances",
                        to="structuretimers.stagingsystem",
                    ),
                ),
                (
                    "timer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="distances",
                        to="structuretimers.timer",
                    ),
                ),
            ],
        ),
        migrations.AddConstraint(
            model_name="distancesfromstaging",
            constraint=models.UniqueConstraint(
                fields=("timer", "staging_system"), name="fpk_distances_from_staging"
            ),
        ),
    ]



from django.test import TestCase
from eveuniverse.tools.testdata import ModelSpec, create_testdata

from . import test_data_filename


class CreateEveUniverseTestData(TestCase):
    def test_create_testdata(self):
        testdata_spec = [
            ModelSpec("EveType", ids=[35832, 35825, 35835]),
            ModelSpec("EveSolarSystem", ids=[30004984, 30045339]),
        ]
        create_testdata(testdata_spec, test_data_filename())



from datetime import timedelta
from unittest.mock import Mock, patch

from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse
from django.utils.timezone import now

from app_utils.testing import (
    create_user_from_evecharacter,
    json_response_to_dict,
    json_response_to_python,
)

from ..models import Timer
from . import (
    LoadTestDataMixin,
    add_permission_to_user_by_name,
    create_fake_staging_system,
    create_fake_timer,
    create_test_user,
)

MODELS_PATH = "structuretimers.models"


@patch(MODELS_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
class TestViewBase(LoadTestDataMixin, TestCase):
    @patch(MODELS_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
    def setUp(self):
        # user
        self.user_1 = create_test_user(self.character_1)
        self.user_2 = create_test_user(self.character_2)
        self.user_2 = add_permission_to_user_by_name(
            "structuretimers.manage_timer", self.user_2
        )
        self.user_3 = create_test_user(self.character_3)

        # timers
        self.timer_1 = create_fake_timer(
            structure_name="Timer 1",
            location_details="Near the star",
            date=now() + timedelta(hours=4),
            eve_character=self.character_1,
            eve_corporation=self.corporation_1,
            user=self.user_1,
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            owner_name="Big Boss",
            details_image_url="http://www.example.com/dummy.png",
            details_notes="Some notes",
        )
        self.timer_2 = create_fake_timer(
            structure_name="Timer 2",
            date=now() - timedelta(hours=8),
            eve_character=self.character_1,
            eve_corporation=self.corporation_1,
            user=self.user_1,
            eve_solar_system=self.system_abune,
            structure_type=self.type_raitaru,
            is_important=True,
        )
        self.timer_3 = create_fake_timer(
            structure_name="Timer 3",
            date=now() - timedelta(hours=8),
            eve_character=self.character_1,
            eve_corporation=self.corporation_1,
            user=self.user_1,
            eve_solar_system=self.system_enaluri,
            structure_type=self.type_astrahus,
        )


class TestListData(TestViewBase):
    def _timer_list_data(self, tab_name: str = "current", user: User = None):
        if not user:
            user = self.user_1
        self.client.force_login(user)
        return self.client.get(
            reverse("structuretimers:timer_list_data", args=[tab_name])
        )

    def _timer_list_data_ids(self, tab_name: str = "current", user: User = None) -> set:
        response = self._timer_list_data(tab_name, user)
        self.assertEqual(response.status_code, 200)
        return set(json_response_to_dict(response).keys())

    # def test_timer_list_view_loads(self):
    #     request = self.factory.get(reverse("structuretimers:timer_list"))
    #     request.user = self.user_1
    #     response = views.timer_list(request)
    #     self.assertEqual(response.status_code, 200)

    def test_return_current_timers(self):
        # when
        timer_ids = self._timer_list_data_ids("current")
        # then
        self.assertSetEqual(timer_ids, {self.timer_1.id})

    def test_return_past_timers(self):
        # when
        timer_ids = self._timer_list_data_ids("past")
        # then
        self.assertSetEqual(timer_ids, {self.timer_2.id, self.timer_3.id})

    def test_should_require_permission(self):
        # given
        user, _ = create_user_from_evecharacter(1003)
        self.client.force_login(user)
        # when
        response = self.client.get(
            reverse("structuretimers:timer_list_data", args=["current"])
        )
        # then
        self.assertEqual(response.status_code, 302)

    def test_show_corp_restricted_to_corp_member(self):
        timer_4 = create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_1,
            eve_corporation=self.corporation_1,
            user=self.user_2,
            visibility=Timer.Visibility.CORPORATION,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id, timer_4.id}
        self.assertSetEqual(timer_ids, expected)

    def test_dont_show_corp_restricted_to_non_corp_member(self):
        create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_corporation=self.corporation_3,
            user=self.user_3,
            visibility=Timer.Visibility.CORPORATION,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id}
        self.assertSetEqual(timer_ids, expected)

    def test_show_alliance_restricted_to_alliance_member(self):
        timer_4 = create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_1,
            eve_corporation=self.corporation_1,
            eve_alliance=self.alliance_1,
            user=self.user_2,
            visibility=Timer.Visibility.ALLIANCE,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id, timer_4.id}
        self.assertSetEqual(timer_ids, expected)

    def test_dont_show_alliance_restricted_to_non_alliance_member(self):
        create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_corporation=self.corporation_3,
            eve_alliance=self.alliance_3,
            user=self.user_3,
            visibility=Timer.Visibility.ALLIANCE,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id}
        self.assertSetEqual(timer_ids, expected)

    def test_show_opsec_restricted_to_opsec_member(self):
        self.user_1 = add_permission_to_user_by_name(
            "structuretimers.opsec_access", self.user_1
        )
        timer_4 = create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_corporation=self.corporation_3,
            user=self.user_3,
            is_opsec=True,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id, timer_4.id}
        self.assertSetEqual(timer_ids, expected)

    def test_dont_show_opsec_restricted_to_non_opsec_member(self):
        create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_corporation=self.corporation_3,
            user=self.user_3,
            is_opsec=True,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id}
        self.assertSetEqual(timer_ids, expected)

    def test_dont_show_opsec_corp_restricted_to_opsec_member_other_corp(self):
        self.user_1 = add_permission_to_user_by_name(
            "structuretimers.opsec_access", self.user_1
        )
        create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_corporation=self.corporation_3,
            user=self.user_3,
            is_opsec=True,
            visibility=Timer.Visibility.CORPORATION,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id}
        self.assertSetEqual(timer_ids, expected)

    def test_show_corp_timer_to_creator_of_different_corp(self):
        timer_4 = create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_corporation=self.corporation_3,
            visibility=Timer.Visibility.CORPORATION,
            user=self.user_1,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id, timer_4.id}
        self.assertSetEqual(timer_ids, expected)

    def test_show_alliance_timer_to_creator_of_different_alliance(self):
        timer_4 = create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_alliance=self.alliance_3,
            eve_corporation=self.corporation_3,
            visibility=Timer.Visibility.ALLIANCE,
            user=self.user_1,
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id, timer_4.id}
        self.assertSetEqual(timer_ids, expected)

    def test_can_show_timers_without_user_character_corporation_or_alliance(self):
        timer_4 = create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
        )
        timer_ids = self._timer_list_data_ids()
        expected = {self.timer_1.id, timer_4.id}
        self.assertSetEqual(timer_ids, expected)

    def test_list_for_manager(self):
        timer_ids = self._timer_list_data_ids(user=self.user_2)
        expected = {self.timer_1.id}
        self.assertSetEqual(timer_ids, expected)

    def test_should_include_distances(self):
        # given
        timer = create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_alliance=self.alliance_3,
            eve_corporation=self.corporation_3,
            user=self.user_1,
        )
        staging_system = create_fake_staging_system(
            eve_solar_system=self.system_enaluri,
            light_years=1.2,
            jumps=3,
        )
        self.client.force_login(self.user_1)
        # when
        response = self.client.get(
            reverse("structuretimers:timer_list_data", args=["current"])
            + f"?staging={staging_system.pk}"
        )
        # then
        data = json_response_to_dict(response)
        obj = data[timer.id]
        self.assertEqual(obj["distance_light_years"], 1.2)
        self.assertEqual(obj["distance_jumps"], 3)
        self.assertTrue(obj["distance"])

    def test_should_not_include_distances(self):
        # given
        timer = create_fake_timer(
            structure_name="Timer 4",
            eve_solar_system=self.system_abune,
            structure_type=self.type_astrahus,
            date=now() + timedelta(hours=8),
            eve_character=self.character_3,
            eve_alliance=self.alliance_3,
            eve_corporation=self.corporation_3,
            user=self.user_1,
        )
        staging_system = create_fake_staging_system(
            eve_solar_system=self.system_enaluri
        )
        self.client.force_login(self.user_1)
        # when
        response = self.client.get(
            reverse("structuretimers:timer_list_data", args=["current"])
            + f"?staging={staging_system.pk}"
        )
        # then
        data = json_response_to_dict(response)
        obj = data[timer.id]
        self.assertIsNone(obj["distance_light_years"])
        self.assertIsNone(obj["distance_jumps"])


@patch(MODELS_PATH + "._task_calc_timer_distances_for_all_staging_systems", Mock())
class TestGetTimerData(TestViewBase):
    def test_should_return_timer(self):
        # given
        self.client.force_login(self.user_1)
        # when
        response = self.client.get(
            reverse("structuretimers:get_timer_data", args=[self.timer_1.pk])
        )
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_python(response)
        self.assertEqual(
            data["structure_display_name"],
            'Astrahus "Timer 1" in Abune near Near the star',
        )

    def test_forbidden(self):
        # given
        self.client.force_login(self.user_1)
        self.timer_1.is_opsec = True
        self.timer_1.save()
        # when
        response = self.client.get(
            reverse("structuretimers:get_timer_data", args=[self.timer_1.pk])
        )
        # then
        self.assertEqual(response.status_code, 302)


class TestSelect2Views(LoadTestDataMixin, TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.user_1 = create_test_user(cls.character_1)

    def test_should_return_solar_systems(self):
        # given
        self.client.force_login(self.user_1)
        # when
        response = self.client.get(
            reverse("structuretimers:select2_solar_systems"), data={"term": "abu"}
        )
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_python(response)
        self.assertEqual(data, {"results": [{"id": 30004984, "text": "Abune"}]})

    def test_should_return_empty_solar_system_list(self):
        # given
        self.client.force_login(self.user_1)
        # when
        response = self.client.get(reverse("structuretimers:select2_solar_systems"))
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_python(response)
        self.assertEqual(data, {"results": None})

    def test_should_return_structure_types(self):
        # given
        self.client.force_login(self.user_1)
        # when
        response = self.client.get(
            reverse("structuretimers:select2_structure_types"), data={"term": "ast"}
        )
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_python(response)
        self.assertEqual(data, {"results": [{"id": 35832, "text": "Astrahus"}]})

    def test_should_return_empty_struture_types_list(self):
        # given
        self.client.force_login(self.user_1)
        # when
        response = self.client.get(reverse("structuretimers:select2_structure_types"))
        # then
        self.assertEqual(response.status_code, 200)
        data = json_response_to_python(response)
        self.assertEqual(data, {"results": None})



# EVE Universe IDs
EVE_CATEGORY_ID_STRUCTURE = 65
EVE_GROUP_ID_CONTROL_TOWER = 365
EVE_GROUP_ID_MOBILE_DEPOT = 1246
EVE_GROUP_ID_REFINERY = 1406
EVE_TYPE_ID_CUSTOMS_OFFICE = 2233
EVE_TYPE_ID_IHUB = 32458
EVE_TYPE_ID_TCU = 32226



from django.core.management import call_command
from django.core.management.base import BaseCommand

from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag

from ... import __title__
from ...constants import (
    EVE_CATEGORY_ID_STRUCTURE,
    EVE_GROUP_ID_CONTROL_TOWER,
    EVE_GROUP_ID_MOBILE_DEPOT,
    EVE_TYPE_ID_CUSTOMS_OFFICE,
    EVE_TYPE_ID_IHUB,
    EVE_TYPE_ID_TCU,
)

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


class Command(BaseCommand):
    help = "Preloads data required for this app from ESI"

    def handle(self, *args, **options):
        call_command(
            "eveuniverse_load_types",
            __title__,
            "--category_id",
            str(EVE_CATEGORY_ID_STRUCTURE),
            "--group_id",
            str(EVE_GROUP_ID_CONTROL_TOWER),
            "--group_id",
            str(EVE_GROUP_ID_MOBILE_DEPOT),
            "--type_id",
            str(EVE_TYPE_ID_CUSTOMS_OFFICE),
            "--type_id",
            str(EVE_TYPE_ID_TCU),
            "--type_id",
            str(EVE_TYPE_ID_IHUB),
        )



from django.urls import path

from . import views

app_name = "structuretimers"

urlpatterns = [
    path("", views.TimerListView.as_view(), name="timer_list"),
    path("add/", views.CreateTimerView.as_view(), name="add"),
    path("remove/<int:pk>", views.RemoveTimerView.as_view(), name="delete"),
    path("edit/<int:pk>", views.EditTimerView.as_view(), name="edit"),
    path("copy/<int:pk>", views.CopyTimerView.as_view(), name="copy"),
    path(
        "list_data/<str:tab_name>",
        views.TimerListDataView.as_view(),
        name="timer_list_data",
    ),
    path(
        "get_timer_data/<str:pk>",
        views.TimerDetailDataView.as_view(),
        name="get_timer_data",
    ),
    path(
        "select2_solar_systems/",
        views.Select2SolarSystemsView.as_view(),
        name="select2_solar_systems",
    ),
    path(
        "select2_structure_types/",
        views.Select2StructureTypesView.as_view(),
        name="select2_structure_types",
    ),
]
